"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-template-curly-in-string */
const __1 = require("../../../..");
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    require('dotenv').config({ path: './config/northwind.env' });
    yield __1.orm.init('./config/northwind.yaml');
}));
describe('insert', () => {
    const context = JSON.parse('{}');
    test('normalize', () => {
        expect(__1.orm.normalize('Categories.insert({name:name,description:description})')).toStrictEqual('Categories.insert({name:name,description:description})');
        expect(__1.orm.normalize('Categories.insert()')).toStrictEqual('Categories.insert({name:name,description:description})');
        expect(__1.orm.normalize('Categories.insert([name, description])')).toStrictEqual('Categories.insert({name:name,description:description})');
        expect(__1.orm.normalize('Categories.insert(entity)')).toStrictEqual('Categories.insert({name:entity.name,description:entity.description})');
        expect(__1.orm.normalize('Orders.insert()')).toStrictEqual('Orders.insert({customerId:customerId,employeeId:employeeId,orderDate:orderDate,requiredDate:requiredDate,shippedDate:shippedDate,shipViaId:shipViaId,freight:freight,name:name,address:address,city:city,region:region,postalCode:postalCode,country:country})');
        expect(__1.orm.normalize('Orders.insert().include(p => p.details)')).toStrictEqual('Orders.insert({customerId:customerId,employeeId:employeeId,orderDate:orderDate,requiredDate:requiredDate,shippedDate:shippedDate,shipViaId:shipViaId,freight:freight,name:name,address:address,city:city,region:region,postalCode:postalCode,country:country}).include(p=>p.details.insert({orderId:orderId,productId:productId,unitPrice:unitPrice,quantity:quantity,discount:discount}))');
        expect(__1.orm.normalize('Orders.insert().include(p => [p.details, p.customer])')).toStrictEqual('Orders.insert({customerId:customerId,employeeId:employeeId,orderDate:orderDate,requiredDate:requiredDate,shippedDate:shippedDate,shipViaId:shipViaId,freight:freight,name:name,address:address,city:city,region:region,postalCode:postalCode,country:country}).include(p=>p.details.insert({orderId:orderId,productId:productId,unitPrice:unitPrice,quantity:quantity,discount:discount}))');
        expect(__1.orm.normalize('Categories.insert(() => ({ name: name, description: description }))')).toStrictEqual('Categories.insert({name:name,description:description})');
        expect(__1.orm.normalize('Categories.insert(p=>{name:p.name,description:p.description})')).toStrictEqual('Categories.insert({name:p.name,description:p.description})');
        expect(__1.orm.normalize('Categories.insert(p=> [p.name,p.description])')).toStrictEqual('Categories.insert({name:name,description:description})');
        expect(__1.orm.normalize('Categories.bulkInsert()')).toStrictEqual('Categories.bulkInsert({name:name,description:description})');
        expect(__1.orm.normalize('Orders.bulkInsert().include(p => p.details)')).toStrictEqual('Orders.bulkInsert({customerId:customerId,employeeId:employeeId,orderDate:orderDate,requiredDate:requiredDate,shippedDate:shippedDate,shipViaId:shipViaId,freight:freight,name:name,address:address,city:city,region:region,postalCode:postalCode,country:country}).include(p=>p.details.bulkInsert({orderId:orderId,productId:productId,unitPrice:unitPrice,quantity:quantity,discount:discount}))');
        expect(__1.orm.normalize('Orders.bulkInsert().include(p => [p.details, p.customer])')).toStrictEqual('Orders.bulkInsert({customerId:customerId,employeeId:employeeId,orderDate:orderDate,requiredDate:requiredDate,shippedDate:shippedDate,shipViaId:shipViaId,freight:freight,name:name,address:address,city:city,region:region,postalCode:postalCode,country:country}).include(p=>p.details.bulkInsert({orderId:orderId,productId:productId,unitPrice:unitPrice,quantity:quantity,discount:discount}))');
    });
    test('model', () => {
        expect(__1.orm.model('Categories.insert({name:name,description:description})')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Categories.insert()')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Categories.insert([name, description])')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Categories.insert(entity)')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Orders.insert()')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }]);
        expect(__1.orm.model('Orders.insert().include(p => p.details)')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details[]', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }]);
        expect(__1.orm.model('Orders.insert().include(p => [p.details, p.customer])')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details[]', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }, { 'name': 'customer', 'type': 'Customers', 'children': [{ 'name': 'id', 'type': 'string' }, { 'name': 'name', 'type': 'string' }, { 'name': 'contact', 'type': 'string' }, { 'name': 'phone', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }] }]);
        expect(__1.orm.model('Categories.insert(() => ({ name: name, description: description }))')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Categories.insert(p=>{name:p.name,description:p.description})')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Categories.insert(p=> [p.name,p.description])')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Categories.bulkInsert()')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.model('Orders.bulkInsert().include(p => p.details)')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details[]', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }]);
        expect(__1.orm.model('Orders.bulkInsert().include(p => [p.details, p.customer])')).toStrictEqual([{ 'name': 'id', 'type': 'integer' }, { 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details[]', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }, { 'name': 'customer', 'type': 'Customers', 'children': [{ 'name': 'id', 'type': 'string' }, { 'name': 'name', 'type': 'string' }, { 'name': 'contact', 'type': 'string' }, { 'name': 'phone', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }] }]);
    });
    test('parameters', () => {
        expect(__1.orm.parameters('Categories.insert({name:name,description:description})')).toStrictEqual([{ 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.parameters('Categories.insert()')).toStrictEqual([{ 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.parameters('Categories.insert([name, description])')).toStrictEqual([{ 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.parameters('Categories.insert(entity)')).toStrictEqual([{ 'name': 'entity.name', 'type': 'string' }, { 'name': 'entity.description', 'type': 'string' }]);
        expect(__1.orm.parameters('Orders.insert()')).toStrictEqual([{ 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }]);
        expect(__1.orm.parameters('Orders.insert().include(p => p.details)')).toStrictEqual([{ 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }]);
        expect(__1.orm.parameters('Orders.insert().include(p => [p.details, p.customer])')).toStrictEqual([{ 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }, { 'name': 'customer', 'type': 'Customers', 'children': [{ 'name': 'id', 'type': 'string' }, { 'name': 'name', 'type': 'string' }, { 'name': 'contact', 'type': 'string' }, { 'name': 'phone', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }] }]);
        expect(__1.orm.parameters('Categories.insert(() => ({ name: name, description: description }))')).toStrictEqual([{ 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.parameters('Categories.insert(p=>{name:p.name,description:p.description})')).toStrictEqual([{ 'name': 'p.name', 'type': 'string' }, { 'name': 'p.description', 'type': 'string' }]);
        expect(__1.orm.parameters('Categories.insert(p=> [p.name,p.description])')).toStrictEqual([{ 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.parameters('Categories.bulkInsert()')).toStrictEqual([{ 'name': 'name', 'type': 'string' }, { 'name': 'description', 'type': 'string' }]);
        expect(__1.orm.parameters('Orders.bulkInsert().include(p => p.details)')).toStrictEqual([{ 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }]);
        expect(__1.orm.parameters('Orders.bulkInsert().include(p => [p.details, p.customer])')).toStrictEqual([{ 'name': 'customerId', 'type': 'string' }, { 'name': 'employeeId', 'type': 'integer' }, { 'name': 'orderDate', 'type': 'dateTime' }, { 'name': 'requiredDate', 'type': 'date' }, { 'name': 'shippedDate', 'type': 'date' }, { 'name': 'shipViaId', 'type': 'integer' }, { 'name': 'freight', 'type': 'decimal' }, { 'name': 'name', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }, { 'name': 'details', 'type': 'Orders.details', 'children': [{ 'name': 'orderId', 'type': 'integer' }, { 'name': 'productId', 'type': 'integer' }, { 'name': 'unitPrice', 'type': 'decimal' }, { 'name': 'quantity', 'type': 'decimal' }, { 'name': 'discount', 'type': 'decimal' }] }, { 'name': 'customer', 'type': 'Customers', 'children': [{ 'name': 'id', 'type': 'string' }, { 'name': 'name', 'type': 'string' }, { 'name': 'contact', 'type': 'string' }, { 'name': 'phone', 'type': 'string' }, { 'name': 'address', 'type': 'string' }, { 'name': 'city', 'type': 'string' }, { 'name': 'region', 'type': 'string' }, { 'name': 'postalCode', 'type': 'string' }, { 'name': 'country', 'type': 'string' }] }]);
    });
    test('constraints', () => {
        expect(__1.orm.constraints('Categories.insert({name:name,description:description})')).toStrictEqual({ "entity": "Categories", "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }] });
        expect(__1.orm.constraints('Categories.insert()')).toStrictEqual({ "entity": "Categories", "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }] });
        expect(__1.orm.constraints('Categories.insert([name, description])')).toStrictEqual({ "entity": "Categories", "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }] });
        expect(__1.orm.constraints('Categories.insert(entity)')).toStrictEqual({ "entity": "Categories", "constraints": [] });
        expect(__1.orm.constraints('Orders.insert()')).toStrictEqual({ "entity": "Orders", "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }] });
        expect(__1.orm.constraints('Orders.insert().include(p => p.details)')).toStrictEqual({ "entity": "Orders", "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "children": [{ "entity": "Orders.details", "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }] }] });
        expect(__1.orm.constraints('Orders.insert().include(p => [p.details, p.customer])')).toStrictEqual({ "entity": "Orders", "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "children": [{ "entity": "Orders.details", "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }] }, { "entity": "Customers", "constraints": [{ "message": "Cannot be null property id in entity Customers", "condition": "isNotNull(id)" }, { "message": "Cannot be null property name in entity Customers", "condition": "isNotNull(name)" }] }] });
        expect(__1.orm.constraints('Categories.insert(() => ({ name: name, description: description }))')).toStrictEqual({ "entity": "Categories", "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }] });
        expect(__1.orm.constraints('Categories.insert(p=>{name:p.name,description:p.description})')).toStrictEqual({ "entity": "Categories", "constraints": [] });
        expect(__1.orm.constraints('Categories.insert(p=> [p.name,p.description])')).toStrictEqual({ "entity": "Categories", "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }] });
        expect(__1.orm.constraints('Categories.bulkInsert()')).toStrictEqual({ "entity": "Categories", "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }] });
        expect(__1.orm.constraints('Orders.bulkInsert().include(p => p.details)')).toStrictEqual({ "entity": "Orders", "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "children": [{ "entity": "Orders.details", "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }] }] });
        expect(__1.orm.constraints('Orders.bulkInsert().include(p => [p.details, p.customer])')).toStrictEqual({ "entity": "Orders", "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "children": [{ "entity": "Orders.details", "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }] }, { "entity": "Customers", "constraints": [{ "message": "Cannot be null property id in entity Customers", "condition": "isNotNull(id)" }, { "message": "Cannot be null property name in entity Customers", "condition": "isNotNull(name)" }] }] });
    });
    test('metadata', () => {
        expect(__1.orm.metadata('Categories.insert({name:name,description:description})')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 18 }, "name": "insert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "insert", "alias": "c" });
        expect(__1.orm.metadata('Categories.insert()')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 18 }, "name": "insert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "insert", "alias": "c" });
        expect(__1.orm.metadata('Categories.insert([name, description])')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 18 }, "name": "insert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "insert", "alias": "c" });
        expect(__1.orm.metadata('Categories.insert(entity)')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 18 }, "name": "insert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "entity.name", "type": "string" }, { "name": "entity.description", "type": "string" }], "constraints": [], "values": [], "defaults": [], "clause": "insert", "alias": "c" });
        expect(__1.orm.metadata('Orders.insert()')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 14 }, "name": "insert", "children": [], "type": "any", "entity": "Orders", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "parameters": [{ "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "values": [], "defaults": [], "clause": "insert", "alias": "o" });
        expect(__1.orm.metadata('Orders.insert().include(p => p.details)')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 24 }, "name": "insert", "children": [{ "classtype": "Sentence", "pos": { "ln": 0, "col": 29 }, "name": "insert", "children": [], "type": "any", "entity": "Orders.details", "columns": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "parameters": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }], "values": [], "defaults": [], "clause": "insert", "alias": "o1" }], "type": "any", "entity": "Orders", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "parameters": [{ "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "values": [], "defaults": [], "clause": "insert", "alias": "o" });
        expect(__1.orm.metadata('Orders.insert().include(p => [p.details, p.customer])')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 24 }, "name": "insert", "children": [{ "classtype": "Sentence", "pos": { "ln": 0, "col": 30 }, "name": "insert", "children": [], "type": "any", "entity": "Orders.details", "columns": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "parameters": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }], "values": [], "defaults": [], "clause": "insert", "alias": "o1" }, { "classtype": "Sentence", "pos": { "ln": 0, "col": 41 }, "name": "insert", "children": [], "type": "any", "entity": "Customers", "columns": [{ "name": "id", "type": "string" }, { "name": "name", "type": "string" }, { "name": "contact", "type": "string" }, { "name": "phone", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "parameters": [{ "name": "id", "type": "string" }, { "name": "name", "type": "string" }, { "name": "contact", "type": "string" }, { "name": "phone", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "constraints": [{ "message": "Cannot be null property id in entity Customers", "condition": "isNotNull(id)" }, { "message": "Cannot be null property name in entity Customers", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "insert", "alias": "c" }], "type": "any", "entity": "Orders", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "parameters": [{ "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "values": [], "defaults": [], "clause": "insert", "alias": "o" });
        expect(__1.orm.metadata('Categories.insert(() => ({ name: name, description: description }))')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 18 }, "name": "insert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "insert", "alias": "c" });
        expect(__1.orm.metadata('Categories.insert(p=>{name:p.name,description:p.description})')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 18 }, "name": "insert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "p.name", "type": "string" }, { "name": "p.description", "type": "string" }], "constraints": [], "values": [], "defaults": [], "clause": "insert", "alias": "c" });
        expect(__1.orm.metadata('Categories.insert(p=> [p.name,p.description])')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 18 }, "name": "insert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "insert", "alias": "c" });
        expect(__1.orm.metadata('Categories.bulkInsert()')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 22 }, "name": "bulkInsert", "children": [], "type": "any", "entity": "Categories", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "parameters": [{ "name": "name", "type": "string" }, { "name": "description", "type": "string" }], "constraints": [{ "message": "Cannot be null property name in entity Categories", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "bulkInsert", "alias": "c" });
        expect(__1.orm.metadata('Orders.bulkInsert().include(p => p.details)')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 28 }, "name": "bulkInsert", "children": [{ "classtype": "Sentence", "pos": { "ln": 0, "col": 33 }, "name": "bulkInsert", "children": [], "type": "any", "entity": "Orders.details", "columns": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "parameters": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }], "values": [], "defaults": [], "clause": "bulkInsert", "alias": "o1" }], "type": "any", "entity": "Orders", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "parameters": [{ "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "values": [], "defaults": [], "clause": "bulkInsert", "alias": "o" });
        expect(__1.orm.metadata('Orders.bulkInsert().include(p => [p.details, p.customer])')).toEqual({ "classtype": "Sentence", "pos": { "ln": 0, "col": 28 }, "name": "bulkInsert", "children": [{ "classtype": "Sentence", "pos": { "ln": 0, "col": 34 }, "name": "bulkInsert", "children": [], "type": "any", "entity": "Orders.details", "columns": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "parameters": [{ "name": "orderId", "type": "integer" }, { "name": "productId", "type": "integer" }, { "name": "unitPrice", "type": "decimal" }, { "name": "quantity", "type": "decimal" }, { "name": "discount", "type": "decimal" }], "constraints": [{ "message": "Cannot be null property orderId in entity Orders.details", "condition": "isNotNull(orderId)" }, { "message": "Cannot be null property productId in entity Orders.details", "condition": "isNotNull(productId)" }], "values": [], "defaults": [], "clause": "bulkInsert", "alias": "o1" }, { "classtype": "Sentence", "pos": { "ln": 0, "col": 45 }, "name": "bulkInsert", "children": [], "type": "any", "entity": "Customers", "columns": [{ "name": "id", "type": "string" }, { "name": "name", "type": "string" }, { "name": "contact", "type": "string" }, { "name": "phone", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "parameters": [{ "name": "id", "type": "string" }, { "name": "name", "type": "string" }, { "name": "contact", "type": "string" }, { "name": "phone", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "constraints": [{ "message": "Cannot be null property id in entity Customers", "condition": "isNotNull(id)" }, { "message": "Cannot be null property name in entity Customers", "condition": "isNotNull(name)" }], "values": [], "defaults": [], "clause": "bulkInsert", "alias": "c" }], "type": "any", "entity": "Orders", "columns": [{ "name": "id", "type": "integer", "required": false, "autoIncrement": true }, { "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "parameters": [{ "name": "customerId", "type": "string" }, { "name": "employeeId", "type": "integer" }, { "name": "orderDate", "type": "dateTime" }, { "name": "requiredDate", "type": "date" }, { "name": "shippedDate", "type": "date" }, { "name": "shipViaId", "type": "integer" }, { "name": "freight", "type": "decimal" }, { "name": "name", "type": "string" }, { "name": "address", "type": "string" }, { "name": "city", "type": "string" }, { "name": "region", "type": "string" }, { "name": "postalCode", "type": "string" }, { "name": "country", "type": "string" }], "constraints": [{ "message": "Cannot be null property customerId in entity Orders", "condition": "isNotNull(customerId)" }, { "message": "Cannot be null property employeeId in entity Orders", "condition": "isNotNull(employeeId)" }], "values": [], "defaults": [], "clause": "bulkInsert", "alias": "o" });
    });
    test('getInfo', () => {
        expect(__1.orm.plan('Categories.insert({name:name,description:description})', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES(?,?)" });
        expect(__1.orm.plan('Categories.insert()', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES(?,?)" });
        expect(__1.orm.plan('Categories.insert([name, description])', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES(?,?)" });
        expect(__1.orm.plan('Categories.insert(entity)', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES(?,?)" });
        expect(__1.orm.plan('Orders.insert()', { stage: 'MySQL' })).toStrictEqual({ "entity": "Orders", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Orders(CustomerID,EmployeeID,OrderDate,RequiredDate,ShippedDate,ShipVia,Freight,ShipName,ShipAddress,ShipCity,ShipRegion,ShipPostalCode,ShipCountry) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)" });
        expect(__1.orm.plan('Orders.insert().include(p => p.details)', { stage: 'MySQL' })).toStrictEqual({ "entity": "Orders", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Orders(CustomerID,EmployeeID,OrderDate,RequiredDate,ShippedDate,ShipVia,Freight,ShipName,ShipAddress,ShipCity,ShipRegion,ShipPostalCode,ShipCountry) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)", "children": [{ "entity": "Orders.details", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO `Order Details`(OrderID,ProductID,UnitPrice,Quantity,Discount) VALUES(?,?,?,?,?)" }] });
        expect(__1.orm.plan('Orders.insert().include(p => [p.details, p.customer])', { stage: 'MySQL' })).toStrictEqual({ "entity": "Orders", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Orders(CustomerID,EmployeeID,OrderDate,RequiredDate,ShippedDate,ShipVia,Freight,ShipName,ShipAddress,ShipCity,ShipRegion,ShipPostalCode,ShipCountry) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)", "children": [{ "entity": "Orders.details", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO `Order Details`(OrderID,ProductID,UnitPrice,Quantity,Discount) VALUES(?,?,?,?,?)" }, { "entity": "Customers", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Customers(CustomerID,CompanyName,ContactName,ContactTitle,Address,City,Region,PostalCode,Country) VALUES(?,?,?,?,?,?,?,?,?)" }] });
        expect(__1.orm.plan('Categories.insert(() => ({ name: name, description: description }))', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES(?,?)" });
        expect(__1.orm.plan('Categories.insert(p=>{name:p.name,description:p.description})', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES(?,?)" });
        expect(__1.orm.plan('Categories.insert(p=> [p.name,p.description])', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES(?,?)" });
        expect(__1.orm.plan('Categories.bulkInsert()', { stage: 'MySQL' })).toStrictEqual({ "entity": "Categories", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Categories(CategoryName,Description) VALUES ?" });
        expect(__1.orm.plan('Orders.bulkInsert().include(p => p.details)', { stage: 'MySQL' })).toStrictEqual({ "entity": "Orders", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Orders(CustomerID,EmployeeID,OrderDate,RequiredDate,ShippedDate,ShipVia,Freight,ShipName,ShipAddress,ShipCity,ShipRegion,ShipPostalCode,ShipCountry) VALUES ?", "children": [{ "entity": "Orders.details", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO `Order Details`(OrderID,ProductID,UnitPrice,Quantity,Discount) VALUES ?" }] });
        expect(__1.orm.plan('Orders.bulkInsert().include(p => [p.details, p.customer])', { stage: 'MySQL' })).toStrictEqual({ "entity": "Orders", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Orders(CustomerID,EmployeeID,OrderDate,RequiredDate,ShippedDate,ShipVia,Freight,ShipName,ShipAddress,ShipCity,ShipRegion,ShipPostalCode,ShipCountry) VALUES ?", "children": [{ "entity": "Orders.details", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO `Order Details`(OrderID,ProductID,UnitPrice,Quantity,Discount) VALUES ?" }, { "entity": "Customers", "dialect": "MySQL", "source": "MySQL", "sentence": "INSERT INTO Customers(CustomerID,CompanyName,ContactName,ContactTitle,Address,City,Region,PostalCode,Country) VALUES ?" }] });
    });
});
//# sourceMappingURL=insert.test.js.map